/*---------------------------------------------------------------------------*
 *
 *	buttonbox
 *	---------
 *
 *	Part of some silly app written by David Allen <s2mdalle@titan.vcu.edu>
 *	http://opop.nols.com/
 *
 *	Released under the terms of the GNU General Public License.
 *
 *	modified by Hellmuth Michaelis <hm@hcs.de>
 *	http://www.freebsd-support.de/misc/
 *
 *	last edit-date: [Thu Aug 17 17:02:54 2000]
 * 
 *---------------------------------------------------------------------------*/

#include "header.h"

static void create_main_window(void);
static void exit_program(GtkWidget * emitter, gpointer data);

void
usage(char *f)
{
	fprintf(stderr, "Usage:  %s\n", f);
	fprintf(stderr,	"You may use %s by itself in which case it will read your\n", f);
	fprintf(stderr, "preferences from $HOME/.buttonboxrc, or you can call %s\n", f);
	fprintf(stderr, "with one argument corresponding to the file to read\n");
	fprintf(stderr, "instead.\n\n");
	fprintf(stderr,	"Of course, you could just type %s --help and get this\n", f);
	fprintf(stderr, "message again, but what the point of that be?\n\n");
	fprintf(stderr, "Written by David Allen <s2mdalle@titan.vcu.edu>\n");
	fprintf(stderr, "http://opop.nols.com/\n");
	fprintf(stderr,	"\nReleased under the terms of the GNU General Public License.\n");
	fprintf(stderr,	"See http://www.gnu.org/ or COPYING for more details.\n\n");
	fprintf(stderr, "Modified by Hellmuth Michaelis<hm@hcs.de>\n");
	fprintf(stderr, "http://www.freebsd-support.de/misc/\n");
}

int
main(int argc, char *argv[])
{
	FILE *fp;
	char filename[2048];
	char *home = getenv("HOME");

	gtk_init(&argc, &argv);

	if (home)
		sprintf(filename, "%s/.buttonboxrc", home);
	else
		sprintf(filename, ".buttonboxrc");

	if (argc > 1)
	{
		if (strcmp(argv[1], "--help") == 0 ||
		    strcmp(argv[1], "-h") == 0 ||
		    strcmp(argv[1], "--version") == 0 ||
		    strcmp(argv[1], "-v") == 0)
		{
			usage(argv[0]);
			exit(0);
		}

#if DEBUG		
		fprintf(stderr, "Opening \"%s\" for reading...\n", argv[1]);
#endif

		fp = fopen(argv[1], "r");
		if (!fp)
		{
			fprintf(stderr, "Couldn't open %s for reading: %s\n",
				argv[1], strerror(errno));
			exit(1);
		}

#if DEBUG
		fprintf(stderr, "File successfully openend.\n");
#endif
	}
	else if (filename)
	{
#if DEBUG
		fprintf(stderr, "Opening \"%s\" for reading...\n", filename);
#endif

		fp = fopen(filename, "r");
		if (!fp)
		{
			fprintf(stderr, "Couldn't open %s for reading: %s\n",
				filename, strerror(errno));
			exit(1);
		}
#if DEBUG
		fprintf(stderr, "File successfully opened.\n");
#endif		
	}
	else
	{
		fprintf(stderr, "No valid file found, exit!\n");
		exit(1);
	}

	window = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_widget_realize(window);

	build_menu();

	/* Do all the *important* stuff.  :) */
	parse_input(fp);

	/* Only after all creation is done */
	create_main_window();
	gtk_main();
	return (0);
}

void
about(GtkWidget * emitter, gpointer data)
{
	GtkWidget *popup = gtk_dialog_new();
	GtkWidget *ok = gtk_button_new_with_label("OK");
	GtkWidget *label = gtk_label_new("ButtonBox");
	GtkWidget *l2 = gtk_label_new("Written by David Allen <s2mdalle@titan.vcu.edu>");
	GtkWidget *l3 = gtk_label_new("http://opop.nols.com/");
	GtkWidget *l4 =	gtk_label_new("Released under the terms of the\nGNU General Public License\nSee http://www.gnu.org/ for more details.");
	GtkWidget *l5 =	gtk_label_new("Modified by Hellmuth Michaelis<hm@hcs.de>");
	GtkWidget *l6 = gtk_label_new("http://www.freebsd-support.de/misc/");

	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->vbox), label,
			   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->vbox), l2,
			   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->vbox), l3,
			   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->vbox), l4,
			   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->vbox), l5,
			   FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->vbox), l6,
			   FALSE, FALSE, 0);

	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
			   GTK_SIGNAL_FUNC(smack), popup);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(popup)->action_area), ok,
			   FALSE, FALSE, 0);

	gtk_window_set_title(GTK_WINDOW(popup), "About ButtonBox");

	gtk_window_set_position(GTK_WINDOW(popup), GTK_WIN_POS_CENTER);

	gtk_widget_show_all(popup);

}

void
build_menu(void)
{
	GtkWidget *item;

	menu = gtk_menu_new();

	item = gtk_menu_item_new_with_label("About");
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			   GTK_SIGNAL_FUNC(about), NULL);
	gtk_menu_append(GTK_MENU(menu), item);

	item = gtk_menu_item_new_with_label("Quit");
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			   GTK_SIGNAL_FUNC(exit_program), NULL);
	gtk_menu_append(GTK_MENU(menu), item);

	gtk_widget_show_all(menu);
}

static void
exit_program(GtkWidget * emitter, gpointer data)
{
	gtk_widget_destroy(window);
	gtk_main_quit();
	gtk_exit(0);
}

static void
create_main_window(void)
{
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(exit_program), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			   GTK_SIGNAL_FUNC(exit_program), NULL);

	gtk_widget_show_all(window);

	/* Move the window around */
	if (final_resting_place.x != -1)
		gdk_window_move(window->window, final_resting_place.x,
				final_resting_place.y);
}

GtkWidget *
xpm_file_to_widget(GtkWidget * foo, gchar * filename)
{
	/* GtkWidget is the storage type for widgets */
	GtkWidget *pixmapwid = NULL;
	GdkPixmap *pixmap = NULL;
	GdkBitmap *mask;
	GtkStyle *style;

	style = gtk_widget_get_style(window);

	/* In order for this to not create a warning, window has to be a
	 * gtk_realize_widget (realized) widget
	 */

	pixmap = gdk_pixmap_create_from_xpm(window->window, &mask,
					    &style->bg[GTK_STATE_NORMAL],
					    filename);
	if (!pixmap)
		return (NULL);

	/* a pixmap widget to contain the pixmap */
	pixmapwid = gtk_pixmap_new(pixmap, mask);

	return (pixmapwid);
}

gint
button_press(GtkWidget * widget, GdkEvent * event)
{
	if (event->type == GDK_BUTTON_PRESS)
	{
		if (event->button.button == RIGHT_MOUSE_BUTTON)
		{
			GdkEventButton *bevent = (GdkEventButton *) event;

			gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
				       NULL, NULL, bevent->button,
				       bevent->time);

			/* Tell calling code that we have handled this event;
			 * the buck stops here.
			 */
			return TRUE;
		}
		else
			return FALSE;
	}

	return FALSE;
}

void
need_destroy(GtkWidget * input)
{
	if (!input || input == (GtkWidget *) NULL)
		return;

	if (GTK_IS_WIDGET(input->parent->parent->parent))
		gtk_widget_destroy(input->parent->parent->parent);
}

void
parse_input(FILE * input)
{
	GtkWidget *action_button;
	GtkWidget *button;

	char *leadval;
	char *auxptr;

	char buffer[BUFLEN];
	char icon_location[BUFLEN];
	char command[BUFLEN];

	icon_location[0] = '\0';

	while (!feof(input))
	{
		FILE_readline(input, buffer, (2048 * sizeof(char)));

		if (*buffer == '#' || isspace(*buffer))
			continue;

		leadval = strtok(buffer, " \t");

		if (leadval == NULL)
			continue;

		auxptr = strtok(NULL, " \t");
		if (auxptr == NULL)
			continue;

		if (!strcmp(leadval, "position"))
		{
			if (!auxptr)
			{
				fprintf(stderr, "Invalid \"position\" statement.\n");
				continue;
			}

			final_resting_place.x = atoi(auxptr);

			auxptr = strtok(NULL, " \t");

			if (!auxptr)
			{
				fprintf(stderr,	"Invalid \"position\" statement.\n");
				continue;
			}

			final_resting_place.y = atoi(auxptr);
		}
		else if (!strcmp(leadval, "icons"))
		{
			strcpy(icon_location, auxptr);
			continue;
		}
		else if (!strcmp(leadval, "orientation"))
		{
#if DEBUG
			fprintf(stderr, "Creating item packing box:	");
#endif
			if (!strcmp(auxptr, "horizontal"))
			{
				if (BOX)
					gtk_widget_destroy(BOX);

#if DEBUG
				fprintf(stderr, "Horizontal.\n");
#endif

				BOX = gtk_hbox_new(FALSE, 0);

				gtk_container_add(GTK_CONTAINER(window), BOX);

				gtk_widget_realize(BOX);

			}
			else if (!strcmp(auxptr, "vertical"))
			{
				if (BOX)
					gtk_widget_destroy(BOX);

#if DEBUG
				fprintf(stderr, "Vertical.\n");
#endif

				BOX = gtk_vbox_new(FALSE, 0);

				gtk_container_add(GTK_CONTAINER(window), BOX);
				gtk_widget_realize(BOX);
			}
		}
		else if (!strcmp(leadval, "icon"))
		{
			char *label_text;
			GtkWidget *label;
			GtkWidget *table;

			if (!BOX)
			{
				fprintf(stderr,	"Critical Error: No 'orientation'");
				fprintf(stderr, " statement before 'icon' statements.\n");
				exit(1);
			}

			label_text = auxptr;

			if (!label_text)
			{
#if DEBUG
				fprintf(stderr,	"No label string found for your XPM button\n");
#endif
				continue;
			}

			auxptr = strtok(NULL, " \t");

			table = gtk_table_new(2, 1, FALSE);

			gtk_container_add(GTK_CONTAINER(BOX), table);

			if (icon_location[0])
			{
				char buffer[1025];

				sprintf(buffer, "%s/%s", icon_location,
					auxptr);
				action_button = xpm_file_to_widget(window, buffer);
			}
			else
			{
				action_button = xpm_file_to_widget(window, auxptr);
			}

			if (!action_button)
			{
#if DEBUG
				fprintf(stderr, "There was an error creating your XPM button\n");
#endif
				continue;
			}

			button = gtk_button_new();

			label = gtk_label_new(label_text);

			gtk_container_add(GTK_CONTAINER(button),
					  action_button);

			gtk_table_attach_defaults(GTK_TABLE(table), button, 0,
						  1, 0, 1);

			gtk_table_attach_defaults(GTK_TABLE(table), label, 0,
						  1, 1, 2);

			auxptr = strtok(NULL, " \t");
			strcpy(command, auxptr);

			while ((auxptr = strtok(NULL, " \t")) != NULL)
			{
				strcat(command, " ");
				strcat(command, auxptr);
			}

			/* XXX need take care of malloced memory !!! */

			auxptr = malloc(strlen(command) + 1);
			strcpy(auxptr, command);

			gtk_signal_connect(GTK_OBJECT(button), "clicked",
					   GTK_SIGNAL_FUNC(do_something),
					   auxptr);

			gtk_signal_connect_object(GTK_OBJECT(button),
						  "button_press_event",
						  GTK_SIGNAL_FUNC
						  (button_press),
						  GTK_OBJECT(window));
		}
	}
}

void
do_something(GtkWidget * emitter, gpointer data)
{
	char *command = (char *) data;
	char bigbuf[2048];

	fprintf(stderr, "command = [%s]\n", command);

	if (!strstr(command, "&"))
	{
		fprintf(stderr, "Adding shell background command for you...\n");
		sprintf(bigbuf, "%s &", command);
		system(bigbuf);
	}
	else
	{
		system(command);
	}
}

void
smack(GtkWidget * emitter, gpointer data)
{
	if (data && GTK_IS_WIDGET((GtkWidget *) data))
		gtk_widget_destroy((GtkWidget *) data);
}

void
FILE_readline(FILE * fp, char *buffer, const int maxlen)
{
	int index = 0;
	int x;
	char tmp;

	if (!fp || feof(fp))
	{
		fprintf(stderr, "FILE_readline Error - attempted read empty/bad stream\n");
		*buffer = '\0';
		return;
	}

	do
	{
		x = fread(&tmp, sizeof(char), 1, fp);

		if ((x == 0) || (tmp == '\n'))
			buffer[index] = '\0';	/* Terminate the string */
		else
			buffer[index++] = tmp;	/* Add the character */

		if (index >= maxlen)
		{
			fprintf(stderr, "File I/O error: premature return readline.\n");
			return;
		}
	}
	while ((x != 0) && (tmp != '\n'));
}
